"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_builder_abstract_1 = require("./query-builder.abstract");
class SQLComparisionBuilder extends query_builder_abstract_1.AbstractQueryBuilder {
    constructor(repo, comparisonMap = SQLComparisionBuilder.DEFAULT_COMPARISON_MAP) {
        super(repo);
        this.comparisonMap = comparisonMap;
        this.paramCount = 0;
    }
    get paramName() {
        const param = `param${this.paramCount}`;
        this.paramCount += 1;
        return param;
    }
    build(field, cmp, val) {
        const col = this.fieldToDbCol(field);
        const normalizedCmp = cmp.toLowerCase();
        if (this.comparisonMap[normalizedCmp]) {
            return this.createComparisonSQL(normalizedCmp, col, val);
        }
        if (normalizedCmp === 'is') {
            return this.isSQLCmp(col, val);
        }
        if (normalizedCmp === 'in') {
            return this.inSQLCmp(col, val);
        }
        if (normalizedCmp === 'notin') {
            return this.notInSQLCmp(col, val);
        }
        throw new Error(`unknown operator "${cmp}"`);
    }
    createComparisonSQL(cmp, col, val) {
        const operator = this.comparisonMap[cmp];
        const { paramName } = this;
        return { sql: `${col} ${operator} :${paramName}`, params: { [paramName]: val } };
    }
    isSQLCmp(col, val) {
        if (val === null) {
            return { sql: `${col} IS NULL`, params: {} };
        }
        if (val === true) {
            return { sql: `${col} IS TRUE`, params: {} };
        }
        if (val === false) {
            return { sql: `${col} IS FALSE`, params: {} };
        }
        throw new Error(`unexpected is operator param ${val}`);
    }
    inSQLCmp(col, val) {
        this.checkNonEmptyArray(val);
        const { paramName } = this;
        return {
            sql: `${col} IN (:...${paramName})`,
            params: { [paramName]: val },
        };
    }
    notInSQLCmp(col, val) {
        this.checkNonEmptyArray(val);
        const { paramName } = this;
        return {
            sql: `${col} NOT IN (:...${paramName})`,
            params: { [paramName]: val },
        };
    }
    checkNonEmptyArray(val) {
        if (!Array.isArray(val)) {
            throw new Error(`Invalid in value expected an array got ${JSON.stringify(val)}`);
        }
        if (!val.length) {
            throw new Error(`Invalid in value expected a non-empty array got ${JSON.stringify(val)}`);
        }
    }
}
exports.SQLComparisionBuilder = SQLComparisionBuilder;
SQLComparisionBuilder.DEFAULT_COMPARISON_MAP = {
    eq: '=',
    neq: '!=',
    gt: '>',
    gte: '>=',
    lt: '<',
    lte: '<=',
    like: 'LIKE',
    notlike: 'NOT LIKE',
    ilike: 'ILIKE',
    notilike: 'NOT ILIKE',
};
//# sourceMappingURL=sql-comparison.builder.js.map